#!/usr/bin/python
import os
import sys
import argparse
from flcli.fl3.auth import Auth
from flcli.fl3.hdfs import File
from flcli.fl3.oozie import OozieClient


def login(args):
    print 'login... user=%s pw=%s domain=%s' % (args.user, args.pw, args.domain)
    auth = Auth()
    auth.login(args.domain, args.user, args.pw)
    auth.save_cookie()

    if auth.validate():
        print 'Success'
    else:
        print 'Fail'
        exit(1)


def upload(args):
    print 'uploading... source=%s dest=%s permission=%s own=%s group=%s' % (args.source, args.dest, args.permission, args.own, args.group)
    auth = Auth()
    auth.load_cookie()

    file = File(auth)
    r = file.upload_file(args.source, args.dest, True)

    if args.own or args.group or args.permission:
        file.permission(args.dest, owner=args.own, group=args.group, permission=args.permission)

    if r:
        print 'Success'
    else:
        print 'Fail'
        exit(1)


def oozie_run(args):
    print 'oozie... type=%s xml_path=%s config=%s base_url=%s' % (args.type, args.xml_path, args.config, args.base_url)

    configuration = {}

    f = open(args.config, 'r')
    for line in f.read().split('\n'):
        # print line
        key = line.split('=')[0].strip()
        value = line.split('=')[1].strip()
        print key, value
        configuration[key] = value

    f.close()

    oozie = OozieClient('http://master2.exem.oss:11000/oozie')

    if args.type == 'wf':
        job_id = oozie.run_workflow(args.xml_path, configuration=configuration)
        print (job_id)
    else:
        job_id = oozie.run_coordinator(args.xml_path, configuration=configuration)
        print (job_id)


def main():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    # login
    parser_hdfs = subparsers.add_parser('login')
    parser_hdfs.add_argument('-user', help="Flamingo WEB username", required=True)
    parser_hdfs.add_argument('-pw', help="Flamingo WEB password", required=True)
    parser_hdfs.add_argument('-domain', help="Flamingo WEB domain", required=True)
    parser_hdfs.set_defaults(func=login)

    # upload
    parser_hdfs = subparsers.add_parser('upload')
    parser_hdfs.add_argument('-source', help="local filename (source)", required=True)
    parser_hdfs.add_argument('-dest', help="HDFS filename (destination)", required=True)
    parser_hdfs.add_argument('-permission', help="Permission (644|755)", default="644")
    parser_hdfs.add_argument('-own', help="Owner username 'hdfs", default="hdfs")
    parser_hdfs.add_argument('-group', help="Group name 'hdfs", default="hdfs")
    parser_hdfs.set_defaults(func=upload)

    # oozie
    parser_hdfs = subparsers.add_parser('oozie-run')
    parser_hdfs.add_argument('-type', help="workflow, coordinator, bundle (wf|cd|bd)", required=True)
    parser_hdfs.add_argument('-xml_path', help="xml path in HDFS", required=True)
    parser_hdfs.add_argument('-config', help="configuration file name", required=True)
    parser_hdfs.add_argument('-base_url', help="Oozie server base url", default="http://master2.exem.oss:11000/oozie")
    parser_hdfs.set_defaults(func=oozie_run)

    parser.parse_args()
    args = parser.parse_args(sys.argv[1:])
    args.func(args)


if __name__ == '__main__':
    main()


import pyoozie


class OozieClient:
    base_url = 'http://localhost:11000/oozie'

    def __init__(self, base_url):
        self.base_url = base_url

    def run_workflow(self, xml_path, configuration):
        c = pyoozie.OozieClient(self.base_url)
        job_id = c.jobs_submit_workflow(xml_path, configuration=configuration, start=True)
        return job_id


    def run_coordinator(self, xml_path, configuration):
        c = pyoozie.OozieClient(self.base_url)
        job_id = c.jobs_submit_coordinator(xml_path, configuration=configuration)
        return job_id



#!/usr/bin/python
import sys
import paramiko
import get_pass
import signal
import time
import select

CONFIG = {}

def print_help():
    print ('*' * 80)
    print (' '*25 + 'Flamingo showlog')
    print ('*' * 80)
    print ('  %-20s\t%-20s\t%s' % ('alias', 'host', 'path'))
    print ('-' * 80)

    for name in sorted(CONFIG.keys()):
        print ('  %-20s\t%-20s\t%s' % (name, CONFIG[name]['host'], CONFIG[name]['path']))
    print ('*' * 80)

    print ('ex)')
    print ('$ python flamingo-cli showlog dev.namenode1')

def view_log(alias):
    info = CONFIG[alias]

    host = info['host']
    user = info['user']
    pw = get_pass.get_pass('%s@%s' % (user, host))

    command = 'tail -f %s' % info['path']

    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    client.connect(host, username=user, password=pw)
    transport = client.get_transport()
    ch = transport.open_session()
    ch.exec_command(command)

    while True:
        try:
            rl, wl, xl = select.select([ch], [], [], 0.0)
            if len(rl) > 0:
                # Must be stdout
                print (ch.recv(1024))
        except:
            print ('close...')
            client.close()
            return


def signal_handler(signal, frame):
    sys.exit(0)

def main_showlog(argv):
    if len(sys.argv) >= 3:
        alias = sys.argv[2]
        if alias in CONFIG:
            view_log(alias)
        else:
            print_help()
    else:
      print_help()

signal.signal(signal.SIGINT, signal_handler)

if __name__ == '__main__':
    main_showlog(sys.argv)


from util import build_headers


class AbstractApi:
    def get_headers(self):
        header_raw = '''
            Host: %s
            Connection: keep-alive
            Content-Length: 19542
            X-CSRF-Token:
            Origin: http://%s
            X-Requested-With: XMLHttpRequest
            User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.84 Safari/537.36
            Content-Type: application/json; charset=UTF-8;
            Accept: */*
            Referer: http://%s/
            Accept-Encoding: gzip, deflate
            Accept-Language: ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7
            Cookie: %s
        ''' % (self.domain, self.domain, self.domain, self.cookie)

        return build_headers(header_raw)


import requests


def build_headers(raw):
    headers = {}

    for line in raw.split('\n'):
        line = line.strip()
        if len(line) > 0:
            t = line.lstrip().split(':')
            headers[t[0]] = ':'.join(t[1:]).lstrip()

    return headers


def get_filename(filename):
    return filename.split('/')[-1]


def fetch_post(url, data, headers):
    r = requests.post(url, data=data, headers=headers)
    # print '\nurl=%s\n%s' % (url, r.content)
    # pprint.pprint(headers)
    return r



#!/usr/bin/python
import sys
import signal
import socket

CONFIG = {}

def print_help():
    print ('*' * 80)
    print (' '*25 + 'Flamingo status')
    print ('*' * 80)
    print ('  %-20s\t%s:%s' % ('alias', 'host', 'port'))
    print ('-' * 80)

    for name in sorted(CONFIG.keys()):
        print ('  %-20s\t%s:%s' % (name, CONFIG[name]['host'], CONFIG[name]['port']))
    print ('*' * 80)

    print ('ex)')
    print ('$ python cli.py status dev.web')
    print ('$ python cli.py status-all')

def status(alias):
    info = CONFIG[alias]

    host = info['host']
    port = info['port']

    result, cause = test_port_scan(host, port)

    if result:
        print ('[%s] %-25s %s' % ('OK', alias, host+':'+port))
    else:
        print ('[%s] %-25s %s (%s)' % ('Fail', alias, host+':'+port, cause))

# import subprocess
# ping_result = {}
# def test_ping(host):
#     if host in ping_result:
#         result = ping_result[host]
#     else:
#         result = subprocess.Popen(['ping', '-c', '1','-t', '1', host], stdout=subprocess.PIPE).stdout.read()
#         # result = os.system('ping -c 1 -t 1 %s' % host)
#
#     ping_result[host] = result
#
#     return result == 0

def test_port_scan(host, port):
    result = False
    cause = ''
    sock = None

    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        errorno = sock.connect_ex((host, int(port)))
        sock.send ('test'.encode())

        if errorno == 0:
            result = True

        else:
            result = False

        sock.close()

    except KeyboardInterrupt:
        cause = "You pressed Ctrl+C"
        if sock is not None: sock.close()

    except socket.gaierror:
        cause = 'Hostname could not be resolved. Exiting'
        if sock is not None: sock.close()

    except socket.error:
        cause = "Couldn't connect to server"
        if sock is not None: sock.close()

    return result, cause

def signal_handler(signal, frame):
    sys.exit(0)

def main_status(argv):
    # python ssh.py arg1 arg2 -> ['ssh.py', 'arg1', 'arg2']

    if len(argv) >= 2:
        alias = argv[1]

        if alias in CONFIG:
            status(alias)
        else:
            print_help()
    else:
      print_help()

def main_status_all(argv):
    for s in sorted(CONFIG):
        status(s)

signal.signal(signal.SIGINT, signal_handler)

if __name__ == '__main__':
    main_status(sys.argv)




import urllib
import requests
import json
import re
import os
import time
from abs import AbstractApi

'''
f = fl3.File(domain='flamingo.exem-oss.org')
f.login('user', 'pw')
f.delete('/user/hive/password/README.md')
f.upload_file('README.md', '/user/hive/password/README.md')

# TO BE
f.chmod('/user/hive/password/README.md', '755')
f.chown('/user/hive/password/README.md', 'hdfs:hdfs')
'''

COOKIE_FILE = os.path.join(str(os.getenv("HOME")), '.fladm_cookie.json')


class Auth(AbstractApi):
    def __init__(self):
        pass

    domain = 'flamingo.exem-oss.org'
    cookie = None
    expired = None

    def exits_cookie(self):
        return os.path.exists(COOKIE_FILE)

    def validate(self):
        if self.exits_cookie():
            self.load_cookie()
            now = time.time()

            if now > self.expired:
                return False

        return bool(re.match('JSESSIONID=[a-zA-Z_0-9]{32}', self.cookie))

    def save_cookie(self):
        f = open(COOKIE_FILE, 'w')
        f.write(json.dumps({
            'domain': self.domain,
            'cookie': self.cookie,
            'expired': time.time() + 600
        }))
        f.close()

    def load_cookie(self):
        f = open(COOKIE_FILE, 'r')
        o = json.loads(f.read())
        self.domain = o['domain']
        self.cookie = o['cookie']
        self.expired = o['expired']

    def login(self, domain, username, password):
        self.domain = domain

        headers = {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
            # 'Accept-Encoding': 'gzip, deflate',
            'Accept-Language': 'ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            # 'Content-Length': '28',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Host': self.domain,
            'Origin': 'http://%s' % self.domain,
            'Pragma': 'no-cache',
            'Referer': 'http://%s/login' % self.domain,
            # 'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.84 Safari/537.36'
        }

        url = 'http://%s/login' % self.domain

        data = urllib.urlencode({
            'username': username,
            'password': password
        })

        s = requests.session()
        s.headers.update(headers)

        s.get(url)
        r = s.post(url, data=data, allow_redirects=False)

        cookie = 'JSESSIONID=%s' % r.cookies.get('JSESSIONID')

        # set cookie to server
        headers['Cookie'] = cookie
        s.headers.update(headers)
        s.get(url)

        self.cookie = cookie
        return cookie



#!/usr/bin/python
import os
import sys
import json
import showlog
import service
import status

CONFIG = json.load(open(os.path.join(os.path.dirname(__file__), 'flamingo-cli-config.json')))

def print_help():
    print ('*' * 80)
    print (' ' * 24 + 'Flamingo Admin CLI')
    print ('*' * 80)
    print (' Available commands')
    print ('-' * 80)
    print ('  showlog\t\t\tshow log file')
    print ('  service\t\t\tservice operations [start|stop|restart]')
    print ('  status\t\t\tshow service status')
    print ('  status-all\t\t\tshow all service status')
    print ('*' * 80)

    print ('examples)')
    print ('\t' + '$ fladm showlog dev.web')
    print ('\t' + '$ fladm service dev.web restart')
    print ('\t' + '$ fladm status dev.web')

def main():
    if (len(sys.argv) >= 2):

        command = sys.argv[1]

        if command == 'showlog':
            showlog.CONFIG = CONFIG['showlog']
            showlog.main_showlog(sys.argv[1:])

        elif command == 'service':
            service.CONFIG = CONFIG['service']
            service.main_service(sys.argv[1:])

        elif command == 'status':
            status.CONFIG = CONFIG['status']
            status.main_status(sys.argv[1:])

        elif command == 'status-all':
            status.CONFIG = CONFIG['status']
            status.main_status_all(sys.argv[1:])

        else:
            print_help()

    else:
        print_help()

if __name__ == '__main__':
    main()


from requests_toolbelt.multipart.encoder import MultipartEncoder
import json
from abs import AbstractApi
from util import fetch_post
from util import get_filename


class File(AbstractApi):
    auth = None
    domain = None
    cookie = None

    def __init__(self, auth):
        self.auth = auth
        self.domain = auth.domain
        self.cookie = auth.cookie

    def delete(self, filename):
        headers = self.get_headers()
        url = 'http://%s/fs/hdfs/delete.json' % self.domain

        data = json.dumps({
            'currentPath': '/',
            'deleteList': [{
                'fullyQualifiedPath': filename,
                'directory': False
            }]
        })

        r = fetch_post(url, data=data, headers=headers)

        return json.loads(r.content)['success']

    def permission(self, filename, group="hdfs", owner="hdfs", permission="644"):
        headers = self.get_headers()
        url = 'http://%s/fs/hdfs/setPermission' % self.domain

        data = json.dumps({
            'currentPath': filename,
            'fileNames': get_filename(filename),
            'fileStatus': 'FILE',
            'files': filename,
            'group': group,
            'owner': owner,
            'permission': permission,
            'recursiveOwner': 0,
            'recursivePermission': 0
        })

        r = fetch_post(url, data=data, headers=headers)

        return json.loads(r.content)['success']

    def upload_file(self, filename, dest_filename, overwrite=False):
        if overwrite:
            self.delete(dest_filename)

        headers = self.get_headers()
        url = 'http://%s/fs/hdfs/upload.json' % self.domain

        dstPath = '/'.join(dest_filename.split('/')[0:-1])
        fileName = get_filename(dest_filename)

        form_data = MultipartEncoder(fields={
            'dstPath': dstPath,
            'fileName': (fileName, open(filename, 'rb'))
        })

        headers['Content-Type'] = form_data.content_type

        r = fetch_post(url, data=form_data, headers=headers)

        return json.loads(r.content)['success']



#!/usr/bin/python
import sys
import paramiko
import get_pass
import signal
import time

CONFIG = {}

def print_help():
    print ('*' * 80)
    print (' '*25 + 'Flamingo service')
    print ('*' * 80)
    print ('  %-20s\t%-20s\t%s' % ('alias', 'host', 'path'))
    print ('-' * 80)

    for name in sorted(CONFIG.keys()):
        print ('  %-20s\t%-20s\t%s' % (name, CONFIG[name]['host'], CONFIG[name]['path']))
    print ('*' * 80)

    print ('ex)')
    print ('$ python cli.py dev.zeppelin [start|stop|restart]')

def service(alias, op):
    info = CONFIG[alias]

    host = info['host']
    user = info['user']

    if 'password' in info:
        pw = info['password']
    else:
        pw = get_pass.get_pass('%s@%s' % (user, host))

    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(host, username=user, password=pw)

    if type(info[op]) is list:
        commands = info[op]
    else:
        commands = [info[op]]

    commands.insert(0, "cd %s" % info['path'])

    ch = ssh.invoke_shell()
    out = ch.recv(9999)
    print (out)

    try:
        for cmd in commands:
            command = '%s \n' % (cmd)
            # print ("> %s" % command)
            ch.send(command)
            out = ''

            # wait for complete
            while not "#" in out:
                while not ch.recv_ready():
                    time.sleep(0.1)

                out = ch.recv(9999)
                print out,

                time.sleep(0.1)
    finally:
        ssh.close()

def signal_handler(signal, frame):
    sys.exit(0)

def main_service(argv):
    if len(argv) >= 3:
        alias = argv[1]
        op = argv[2]
        if alias in CONFIG:
            service(alias, op)
        else:
            print_help()
    else:
      print_help()

signal.signal(signal.SIGINT, signal_handler)

if __name__ == '__main__':
    main_service(sys.argv)


import io
from setuptools import find_packages, setup


# Read in the README for the long description on PyPI
def long_description():
    return 'description'
    # with io.open('README.rst', 'r', encoding='utf-8') as f:
    #     readme = f.read()
    # return readme

setup(name='fladm',
      version='1.7.0',
      description='Flamingo admin cli tool',
      long_description=long_description(),
      url='http://gitlab.exem-oss.org/flamingo/flamingo-adm-cli.git',
      author='Jaeik Park',
      author_email='jaeikpark81@gmail.com',
      license='Apache2',
      packages=find_packages(),
      install_requires=['paramiko', 'requests', 'requests-toolbelt', 'pyoozie'],
      entry_points = {
        'console_scripts': ['fladm=fladm.cli:main', 'flcli=flcli.cli:main']
      },
      exclude_package_data={
            '': ['flcli/cookie.dat']
      },
      package_data={'': ['*.json']},
      classifiers=[
          'Programming Language :: Python :: 2.7'
      ],
      zip_safe=False)
